$(document).ready(function () {
    $(top).on("unitframeplus_show_promotion_tree", function (_, e) {
        if (App.stonehearthClient) {
            App.stonehearthClient.showPromotionTree(e.entity);
        }
    });
});

$(top).on('stonehearthReady', function (cc) {
    if (!App.gameView) {
        return;
    }
    var unitFramePlus = App.gameView.getView(App.UnitFramePlusView);
    if (!unitFramePlus) {
        var view = App.gameView.addView(App.UnitFramePlusView, {});
    }
});

App.UnitFramePlusView = App.View.extend({
    templateName: 'unitFramePlus',
    uriProperty: 'model'
});

App.StonehearthUnitFrameView.reopen({
    components: {
        "stonehearth:ai": {
            "status_text_data": {}
        },
        "stonehearth:attributes": {
            "attributes": {}
        },
        "stonehearth:building": {},
        "stonehearth:fabricator": {},
        "stonehearth:incapacitation": {
            "sm": {}
        },
        "stonehearth:item_quality": {
        },
        "stonehearth:commands": {
            "commands": {}
        },
        "stonehearth:job": {
            'curr_job_controller': {}
        },
        "stonehearth:buffs": {
            "buffs": {
                "*": {}
            }
        },
        'stonehearth:expendable_resources': {},
        "stonehearth:unit_info": {},
        "stonehearth:stacks": {},
        "stonehearth:material": {},
        "stonehearth:workshop": {
            "crafter": {},
            "crafting_progress": {},
            "order": {}
        },
        "stonehearth:happiness": {
            "current_mood_buff": {}
        },
        "stonehearth:pet": {},
        "stonehearth:party": {
            "members": {
                "*": {
                    "entity": {
                        "stonehearth:work_order": {
                            "work_order_statuses": {},
                            "work_order_refs": {}
                        }
                    }
                }
            }
        },
        "stonehearth:party_member": {
            "party": {
                "stonehearth:unit_info": {}
            }
        },
        "stonehearth:siege_weapon": {},
        "stonehearth:door": {},
        "stonehearth:iconic_form": {
            "root_entity": {
                "uri": {},
                'stonehearth:item_quality': {},
            }
        },
        "stonehearth:work_order": {
            "work_order_statuses": {},
            "work_order_refs": {}
        }
    },

    _populationUri: null,
    _playerID: "",
    _isHostile: false,
    _partyObserverTimer: null,
    _partyMembersJobEnabledCache: null,

    init: function () {
        var self = this;
        self._super();

        radiant.call('stonehearth:get_population')
           .done(function (response) {
               if (self.isDestroying || self.isDestroyed) {
                   return;
               }
               self._populationUri = response.population;
           });
        self._playerID = App.stonehearthClient.getPlayerId();
    },

    willDestroyElement: function () {
        var self = this;
        self._super();

        if (_partyObserverTimer) {
            Ember.run.cancel(_partyObserverTimer);
        }
    },

    _hostilityObserver: function () {
        var self = this;
        var playerID = self.get('model.player_id');
        if (playerID && playerID != self._playerID) {
            radiant.call('unitframeplus:are_player_ids_hostile', playerID, self._playerID)
                .done(function (e) {
                    self._isHostile = e.are_hostile;
                    self._updateIsHostile();
                });
        }
        else {
            self._isHostile = false;
            self._updateIsHostile();
        }
    }.observes('model'),

    _updateIsHostile: function () {
        var self = this;

        if (self._isHostile) {
            self.$("#info").addClass('hostile');
        }
        else {
            self.$("#info").removeClass('hostile');
        }
        
        // show the "attack with party" button
        self._updatePartyAttackButtons(self._isHostile && self.get('model.stonehearth:expendable_resources.resources.health') != undefined);
    },

    _updateJobToggleButton: function () {
        var self = this;
        var jobOn = false;
        var jobOff = false;

        var members = self.getCitizenMembers();
        var is_combat_class = self.get('model.stonehearth:job.curr_job_controller.is_combat_class') || self.get('model.stonehearth:party');
        radiant.each(members, function (i, member) {
            var isEnabled = self.isJobEnabled(member);
            if (isEnabled != null) {
                if (isEnabled) {
                    jobOn = true;
                }
                else {
                    jobOff = true;
                }
            }
        });

        var jobToggleBtn = self.$("#jobToggleButton");
        if (jobToggleBtn.length < 1) {
            var div = $("#jobToggleDiv");
            div.appendTo(self.$("#unitFrame"));
            jobToggleBtn = self.$("#jobToggleButton");
            jobToggleBtn.on('click', function () {
                self.toggleJob();
            });
        }
        var jobToggleDiv = self.$("#jobToggleDiv");

        if (!jobOn && !jobOff) {
            // no job; hide the button
            jobToggleDiv.hide();
        }
        else {
            var title, tooltip;
            jobToggleDiv.show();
            jobToggleBtn.removeClass('toggledCivOn toggledCivOff toggledOn toggledOff toggledSome');

            if (!is_combat_class) {
                if (!jobOn) {
                    // job off
                    jobToggleBtn.addClass('toggledCivOff');
                    title = 'toggle_on';
                    tooltip = 'toggledCivOff';
                }
                else {
                    // only job on
                    jobToggleBtn.addClass('toggledCivOn');
                    title = 'toggle_off';
                    tooltip = 'toggledCivOn';
                }
            }
            else {
                if (!jobOn) {
                    // only job off
                    jobToggleBtn.addClass('toggledOff');
                    title = 'toggle_on';
                    tooltip = 'toggledOff';
                }
                else if (!jobOff) {
                    // only job on
                    jobToggleBtn.addClass('toggledOn');
                    title = 'toggle_off';
                    tooltip = 'toggledOn';
                }
                else {
                    // both job on and off
                    jobToggleBtn.addClass('toggledSome');
                    title = 'toggle_on';
                    tooltip = 'toggledSome';
                }
            }

            title = i18n.t('unitframeplus:ui.game.unit_frame_plus.job_toggle.' + title);
            tooltip = i18n.t('unitframeplus:ui.game.unit_frame_plus.job_toggle.' + tooltip);
            if(jobToggleDiv.hasClass('tooltipstered')){
                jobToggleDiv.tooltipster('destroy');
            }
            jobToggleDiv.tooltipster({
                content: $(App.tooltipHelper.createTooltip(title, tooltip))
            });
        }
    },

    _updatePartyAttackButtons: function (visible) {
        var self = this;
        var partyAttackButtons = self.$("#partyAttackButtons");
        if (partyAttackButtons.length < 1) {
            // move the div from the unitframeplus view into this one
            var div = $("#partyAttackButtons");
            div.appendTo(self.$("#unitFrame"));
            partyAttackButtons = self.$("#partyAttackButtons");

            var getImageUrl = function (end) {
                var e = document.getElementById('attackImage');
                var style = getComputedStyle(e);
                var base = style.getPropertyValue('content');
                return base.slice(0, base.lastIndexOf('/') + 1) + 'attackWith' + end + '.png")';
            };

            div.on('mouseleave', function () {
                self.$("#attackImage").css('content', getImageUrl("Parties"));
            });
            self.$('.attackParty')
                .on('click', function () {
                    self.partyAttackClicked(this.getAttribute('attackParty'));
                })
                .on('mouseenter', function () {
                    self.$("#attackImage").css('content', getImageUrl(this.getAttribute('attackParty')));
                });
                //.on('mouseleave', function () {
                //    self.$("#attackImage").css('content', getImageUrl("Parties"));
            //});

            if (div.hasClass('tooltipstered')) {
                div.tooltipster('destroy');
            }
            var title = i18n.t('unitframeplus:ui.game.unit_frame_plus.attack_with_parties.title');
            var tooltip = i18n.t('unitframeplus:ui.game.unit_frame_plus.attack_with_parties.description');
            div.tooltipster({
                content: $(App.tooltipHelper.createTooltip(title, tooltip))
            });
        }

        if (visible) {
            partyAttackButtons.show();
        }
        else {
            partyAttackButtons.hide();
        }
    },

    didInsertElement: function() {
        var self = this;
        this._super();

        self._updateJobToggleButton();
    },

    _partyObserver: function(){
        var self = this;
        Ember.run.once(self, '_partyTimerCheck');
    }.observes('model.stonehearth:party', 'model.stonehearth:party.members'),

    _partyTimerCheck: function(){
        var self = this;
        var party = self.get('model.stonehearth:party');
        if (party && party.members) {
            // we have a party; if we don't have a party cache, create it
            // since we're just now getting the party cache, we just selected a party, so we're already updating
            // through the normal observer; otherwise, compare to existing cache and if different, update

            let newCache = {};
            radiant.each(party.members, function (i, member) {
                newCache[i] = self.isJobEnabled(member.entity);
            });

            if (self._partyMembersJobEnabledCache) {
                // check if the new cache is different from the old cache
                let diff = false;
                if (self._partyMembersJobEnabledCache.length != newCache.length) {
                    diff = true;
                }
                else {
                    radiant.each(newCache, function (i, member) {
                        if (self._partyMembersJobEnabledCache[i] !== member) {
                            diff = true;
                        }
                    });
                }

                if (diff) {
                    self._partyMembersJobEnabledCache = newCache;
                    self._updateJobToggleButton();
                }
            }
            else {
                self._partyMembersJobEnabledCache = newCache;
            }

            // check again in 100ms to see if the party members job status has changed
            self._partyObserverTimer = Ember.run.later(self, '_partyObserver', 100);
        }
        else {
            self._partyObserverTimer = null;
            self._partyMembersJobEnabledCache = null;
        }
    },

    _updateObserver: function(){
        var self = this;
        Ember.run.once(self, '_updateJobToggleButton');
    }.observes('model.stonehearth:work_order', 'model.stonehearth:party.members'),

    getCitizenMembers: function(){
        var self = this;
        var members = [];

        if (App.stonehearthClient.getPlayerId() != self.get('model.player_id')) {
            return members;
        }

        // if it's a party, we'll need to process through each party member
        var party = self.get('model.stonehearth:party');
        if (party) {
            radiant.each(self.get('model.stonehearth:party.members'), function (k, v) {
                members.push({
                    "id": radiant.getEntityId(v.entity),
                    "stonehearth:work_order": v.entity["stonehearth:work_order"]
                });
            });
        }
        else {
            members.push({
                "id": radiant.getEntityId(self.get('model')),
                "stonehearth:work_order": self.get("model.stonehearth:work_order")
            });
        }

        return members;
    },

    isJobEnabled: function (entity) {
        var wo = entity["stonehearth:work_order"];
        if (wo) {
            var workOrders = wo.work_order_statuses;
            var workOrderRefs = wo.work_order_refs;
            if (!workOrders || !workOrderRefs) {
                return null;
            }
            return workOrderRefs['job'] && workOrders['job'] != 'disabled';
        }
        return null;
    },

    toggleJob: function () {
        var self = this;

        if (self.get('model.player_id') == App.stonehearthClient.getPlayerId()) {
            var members = self.getCitizenMembers();

            var div = self.$("#jobToggleButton");
            var checked = !(div.hasClass('toggledOn') || div.hasClass('toggledCivOn'));

            radiant.call('radiant:play_sound', { 'track': 'stonehearth:sounds:ui:action_click' });
            radiant.each(members, function (i, member) {
                radiant.call_obj(self._populationUri, 'change_work_order_command', 'job', member.id, checked);
            });
        }
    },

    partyAttackClicked: function (party) {
        var self = this;

        if (party.toLowerCase() === "all") {
            for (var i = 1; i <= 4; i++) {
                self.issueAttackCommand("party_" + i);
            }
        }
        else {
            self.issueAttackCommand("party_" + party);
        }

        radiant.call('radiant:play_sound', { 'track': 'stonehearth:sounds:ui:action_click' });
    },

    issueAttackCommand: function (party_id) {
        var self = this;
        radiant.call_obj('stonehearth.unit_control', 'get_party_by_population_name', party_id)
            .done(function (response) {
                if (response.result) {
                    radiant.call_obj('stonehearth.combat_server_commands', 'party_attack_target_entity', response.result, self.get('uri'));
                }
            });
    }
});